package edu.msoe.se2890.supplied;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import lejos.nxt.comm.Bluetooth;
import lejos.nxt.comm.NXTConnection;
import lejos.nxt.comm.USB;

public class LegoMindstormsProxy implements Runnable {
	public static final int USBPROXY = 0;
	public static final int BLUETOOTHPROXY = 1;

	private static LegoMindstormsProxy singleton[] = { null, null };
	private ArrayList<Integer> transmitQueue;
	private boolean keepRunning = true;
	private int proxyType;
	private DataOutputStream dOut=null;
	private DataInputStream dIn=null;
	private Object lock = new Object();
	
	/**
	 * This method will return the Singleton instance for the proxy which
	 * communicates with the PC proxy.
	 * 
	 * @param type
	 *            The type is either USBPROXY or BLUETOOTHPROXY.
	 * @return The singleton instance of the proxy will be returned.
	 */
	public synchronized static LegoMindstormsProxy getSingleton(int type) {
		if ((type == USBPROXY) || (type == BLUETOOTHPROXY)) {
			if (singleton[type] == null) {
				singleton[type] = new LegoMindstormsProxy(type);
			}
			return singleton[type];
		} else {
			return null;
		}
	}

	/**
	 * This method will shutdown the running proxy.
	 */
	public void shutdown() {
		keepRunning = false;
	}

	/**
	 * This method will enqueue an integer that is to be transmitted to the pc.
	 * 
	 * @param valueToSend
	 *            This is the value to send.
	 */
	public void enqueue(int valueToSend) {
		this.transmitQueue.add(valueToSend);
	}

	/**
	 * This method will receive an integer from the pc.
	 * 
	 * @return The return value will be an integer from the pc. If there is
	 *         nothing on the queue, the code will block.
	 */
	public int receiveInt() {
		int retVal=Integer.MIN_VALUE;

		try {
			if (dIn == null){
				synchronized(this)
				{
					wait();
				}
			}
			retVal = dIn.readInt();
		} catch (IOException e) {
			// An exception has occurred reading from the pipe.
			// Shutdown the proxy.
			shutdown();
		} catch (InterruptedException e) {
			// Just return with the negative value.
		} 

		return retVal;
	}

	/**
	 * This is the run method. It simply runs the proxy.
	 */
	public void run() {
		this.startProxy();
	}

	/**
	 * This method will query the proxy to determine if it is still running or
	 * not.
	 * 
	 * @return true or false, depending on the state of the proxy.
	 */
	public boolean isRunning() {
		return keepRunning;
	}

	/**
	 * This method will instantiate a new instance of the proxy that
	 * communicates with the remote device.
	 */
	private LegoMindstormsProxy(int type) {
		transmitQueue = new ArrayList<Integer>();
		proxyType = type;
	}

	/**
	 * This method will run the given proxy. It will start a connection with the
	 * remote device and start transmitting null data.
	 */
	private void startProxy() {
		NXTConnection conn = null;

		if (this.proxyType == USBPROXY) {
			conn = USB.waitForConnection();
		} else if (proxyType == BLUETOOTHPROXY) {
			conn = Bluetooth.waitForConnection();
		} else {
		}

		dOut = conn.openDataOutputStream();
		dIn = conn.openDataInputStream();
		synchronized(this)
		{
			notifyAll();
		}
		
		while (keepRunning == true) {
			int i;

			if (transmitQueue.size() > 0) {
				synchronized (lock) {
					i = transmitQueue.get(0);
					transmitQueue.remove(0);
				}
				try {
					dOut.writeInt(i);
					dOut.flush();
				} catch (IOException ioe) {
					System.err.println("IO Exception writing bytes");
				}
			} else {
				i = Integer.MIN_VALUE;
			}
		}
		// Close the open connection.
		conn.close();
	}
}
